#ifndef MASH_CONSOLE_H
#define MASH_CONSOLE_H

#include "rc/console-rc.h"
#include <stdio.h>


class MashConsole {
public:
	MashConsole(int initialBufferSize=8000);
	~MashConsole();
	void Init(HINSTANCE hInstance);
	void Create(int x=CW_USEDEFAULT,
		    int y=CW_USEDEFAULT,
		    int width=CW_USEDEFAULT, int height=CW_USEDEFAULT);
	void Destroy();
	void Show(BOOL showFlag);
	void DoModal();
	void SetTitle(const char *title) {
		if (Exists()) SetWindowText(toplevel_, title);
	}
	
	BOOL Toggle();
	BOOL IsVisible() { return IsWindowVisible(toplevel_); }
	BOOL Exists() { return (toplevel_!=NULL && IsWindow(toplevel_)); }
	BOOL SetBufferSize(int size);
	
private:
	void Output(const char *buffer, int size);
	void CreateSpace(int need);
	void CopyTo(char *newBuffer, int &size);
	
	BOOL CreateOutputPipe(int &outputReadHandle);
	void CloseOutputPipe();
	static void ThreadMain(void *args);
	
	void BuildWindow(HWND toplevel);
	static BOOL CALLBACK AboutDialogProc(HWND hdlg, UINT msg,
					     WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK MainWindowProc_Static(HWND hwnd, UINT msg,
						      WPARAM wParam,
						      LPARAM lParam);
	LRESULT MainWindowProc(HWND hwnd, UINT msg,
			       WPARAM wParam, LPARAM lParam);

	HINSTANCE hInstance_;
	HWND toplevel_, statusBar_, edit_, hide_;

	char *buffer_;
	int len_, maxLen_;


	class SavedFileHandle {
	public:
		SavedFileHandle() : osf_handle_(INVALID_HANDLE_VALUE) { }
		~SavedFileHandle() { }
		
		BOOL Set(DWORD nStdHandle, int newHandle, FILE *file);
		void Restore(DWORD nStdHandle, FILE *file);

	private:
		HANDLE osf_handle_;
		FILE file_;
	};

	SavedFileHandle savedStdOut_, savedStdErr_;

	friend class OTclMashConsole;
};


#endif /* #if MASH_CONSOLE_H */
