#ifndef mash_tcpchan_h
#define mash_tcpchan_h

#include "config.h"
#include "tclcl.h"
#include "iohandler.h"
#include "tk.h"

class TCPServerHandler;
class TCPClientHandler;

class TCPServerChannel : public TclObject {
public:	
	TCPServerChannel();
	~TCPServerChannel();
	int recv(u_char* buf, int len, u_int32_t&, u_int16_t&);
	void send(const u_char* buf, int len);
	void reset(void);
	inline int sock() { return sock_; }
	virtual int command(int argc, const char*const*argv);
	inline void handler(TCPServerHandler* h) { handler_ = h; }
private:
	int opensock(u_int16_t port) const;
	int connsock_;
	int sock_;
	u_int32_t fromaddr_;
	u_int16_t fromport_;
	int concurrent_;
	int parent_;
	TCPServerHandler* handler_;
};

class TCPClientChannel : public TclObject {
public:	
	TCPClientChannel();
	~TCPClientChannel();
	int recv(u_char* buf, int len);
	void send(const u_char* buf, int len);
	inline int sock() { return sock_; }
	virtual int command(int argc, const char*const*argv);
	inline void handler(TCPClientHandler* h) { handler_ = h; }
	void closesock();

private:
	int opensock(u_int32_t addr, u_int16_t port) const;
	int sock_;
	TCPClientHandler* handler_;
};

class TCPClientHandler;
class TCPServerHandler;
	
class TCPClientManager {
public:
	virtual void recv(TCPClientHandler*) = 0;
};

class TCPServerManager {
public:
	virtual void recv(TCPServerHandler*) = 0;
};

class TCPClientHandler : public IOHandler, public TclObject {
public:
	inline TCPClientHandler() : manager_(0), chan_(0) {}
	virtual void dispatch(int) {
		manager_->recv(this);
	}
	inline TCPClientChannel* chan() const { return (chan_); }
	void reset(void) {
		chan(0);
	}
	virtual void chan(TCPClientChannel *c) {
		unlink();
		chan_ = c;
		if (c != 0) {
			c->handler(this);
			link(c->sock(), TCL_READABLE);
		}
	}
	inline int recv(u_char* bp, int len) {
		return (chan_->recv(bp, len));
	}
	inline void send(u_char* bp, int len) {
		chan_->send(bp, len);
	}
	inline void manager(TCPClientManager* sm) { manager_ = sm; }
	virtual int command(int argc, const char*const* argv);
protected:
	TCPClientManager* manager_;
	TCPClientChannel* chan_;
};

class TCPServerHandler : public IOHandler, public TclObject {
public:
	inline TCPServerHandler() : manager_(0), chan_(0) {}
	virtual void dispatch(int) {
		manager_->recv(this);
	}
	inline TCPServerChannel* chan() const { return (chan_); }
	void reset(void) {
		unlink();
		chan_->reset();
		link(chan_->sock(), TCL_READABLE);
	}
	virtual void chan(TCPServerChannel *c) {
		unlink();
		chan_ = c;
		if (c != 0) {
			c->handler(this);
			link(c->sock(), TCL_READABLE);
		}
	}
	inline int recv(u_char* bp, int len, u_int32_t& addr, 
			u_int16_t& port) {
		return(chan_->recv(bp, len, addr, port));
	}
	inline void send(u_char* bp, int len) {
		chan_->send(bp, len);
	}
	inline void manager(TCPServerManager* sm) { manager_ = sm; }
	virtual int command(int argc, const char*const* argv);
protected:
	TCPServerManager* manager_;
	TCPServerChannel* chan_;
};

#endif /* mash_tcpchan_h */
