#
# Copyright (c) @ Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/hm/application-hm.tcl,v 1.44 1998/05/11 23:32:06 schuett Exp $
#
import Application HMAgent

Class HMApplication -superclass Application

HMApplication public init argv {
	$self next hm
	set o [$self options]
	$self init_args $o
	$self init_resources $o
	$o parse_args $argv
	$self init_local
	
	if { [$self get_option doFork] != "" } {
		close stdout
		close stdin
		close stderr
		fork
		set logfd [open [$self get_option logFile] a+]
	} else {
		set logfd stdout
	}
	$self add_option execArgs $argv
	new HMAgent $self $logfd
}

HMApplication private init_args o {
	$o register_boolean_option -lb loadBalance
	$o register_boolean_option -fork doFork
	$o register_boolean_option -glunix glunix
	$o register_boolean_option -noload noLoad
	$o register_boolean_option -nolog noLog

	$o register_option -target targetNum
	$o register_option -int checkInterval
	$o register_option -path execPath
	$o register_option -log logFile
	$o register_option -rcmd execCmd
	$o register_option -conffile megaConfFile
	$o register_option -megactrl megaCtrl
	$o register_option -scriptdir scriptDir
}

HMApplication private init_resources o {

	$o add_default megaCtrl 224.4.5.24/50000/31
	$o add_default megaCtrlBW 20000
	# 60 seconds max startup wait for first message
	$o add_default megaStartupWait 60
	$o add_default scriptDir /var/tmp

	$o add_default unicastOutput 1

	$o add_default lambda 5.0
	$o add_default maxWait 15000
	$o add_default maxPending 5

	$o add_default minPort 10000
	$o add_default maxPorts 10000

	$o add_default checkFactor 2

	$o add_default checkInterval 20000
	$o add_default deathInterval 10000
	$o add_default minDeathWait 60000

	$o add_default highLoad 1.00
	$o add_default lowLoad 0.05
	$o add_default noLoad yes
	$o add_default execPath [pwd]
	$o add_default execCmd "ssh -n"
	$o add_default minHmNum 1
	$o add_default loadSamples 2
	$o add_default logFile /tmp/hmlog
	$o add_default megaConfFile mega.conf
	$o add_default link no
}
