
import RTPApplication
import VideoAgent
import AnnounceListenManager;

Class FrameGen -superclass RTPApplication

FrameGen instproc init args {
    $self next frame_gen

    $self init_resources;

    $self instvar init_load_ 
    $self instvar cur_index_
    $self instvar playout_rate_ playing_flag_;
    $self instvar in_agent_ out_agent_ loading_flag_;
    $self instvar play_buffers_;
    $self instvar seg_length_;
    $self instvar bp_ encoder_;
    $self instvar control_;

    set control_ [new AnnounceListenManager/FrameGenControl [$self get_option mtu] [$self get_option cntrl_spec] $self];

    set init_load_ 0;
    set cur_index_ 0;
    set playout_rate_ 30.0;
    set playing_flag_ 0;
    set loading_flag_ 0;
    set seg_length_ [$self get_option seg_length];

    set in_agent_ [new FrameGenVideoAgent $self [$self get_option input_spec]];
    set out_agent_ [new VideoAgent $self [$self get_option output_spec]];

    for {set i 0} {$i < $seg_length_} {incr i} {
	set play_buffers_($i) [new VidRep/JPEGCompressed];
    }

    set encoder_ [new Module/VideoEncoder/JPEGCompressedToJPEG];
    set ssrc [$out_agent_ get_local_srcid];
    set bp_ [new BufferPool/RTP];
    $bp_ srcid $ssrc;

    $encoder_ buffer-pool $bp_;
    $encoder_ mtu [$self get_option mtu];
    $encoder_ target [$out_agent_ get_transmitter];

    button .load -text "Load" -command "$self load";
    button .play_stop -text "Play" -command "$self play";
    scale .frame_rate -orient horizontal -from 1.0 -to 30.0 -resolution 0.1 -command "$self set_rate ";
    button .quit -text "Quit" -command exit;
    label .host_name -text "Running on [exec hostname]";

    pack .host_name -side bottom -fill both -expand 1;
    pack .quit -side bottom -fill both -expand 1;
    pack .frame_rate -side bottom -fill both -expand 1
    pack .load -side left -fill both -expand 1;
    pack .play_stop -side right -fill both -expand 1;


}

FrameGen instproc set_rate {rate} {
    $self instvar playout_rate_;

    set playout_rate_ $rate;
}

FrameGen instproc init_resources {} {
    $self add_option network ip
    $self add_option mtu 1024
    $self add_option defaultTTL 32
    $self add_option sessionType rtpv2
    $self add_option maxVideoSessionBW 2000000
#    $self add_option input_spec 224.2.3.4/22334
    $self add_option input_spec 239.255.124.168/49256
    $self add_option output_spec 224.2.2.4/22334
    $self add_option seg_length 300
    $self add_option cntrl_spec 224.4.3.2/22334/16
#    $self add_option in_ssrcid 970993913
    $self add_option in_ssrcid *
}

FrameGen instproc stop {} {
    $self instvar next_play_after_handle_;

    if {[info exists $next_play_after_handle_]} {
	after cancel $next_play_after_handle_;
	unset next_play_after_handle_;
    }
    $self instvar playing_flag_;
    set playing_flag_ 0;

    .play_stop configure -text "Play" -command "$self play";
}

FrameGen instproc play {} {
    $self instvar playing_flag_;
    if {$playing_flag_} {
	return;
    }

    $self instvar init_load_;
    if {!$init_load_} {
	puts "Must load first";
	return;
    }

    $self instvar loading_flag_
    if {$loading_flag_} {
	puts "Loading, pleas wait";
	return;
    }

    $self instvar playout_rate_;
    set interval [expr int(1000.0 / $playout_rate_) ];
    $self instvar next_play_after_handle_;
    set next_play_after_handle_ [after $interval $self do_play];

    set playing_flag_ 1;
    .play_stop configure -text "Stop" -command "$self stop";
}

FrameGen instproc do_play {} {
    $self instvar cur_index_ playing_flag_ seg_length_
    $self instvar next_play_after_handle_;
    $self instvar playout_rate_;
    $self instvar encoder_ play_buffers_;

    if {!$playing_flag_} {
	return;
    }

    set ts [expr int([clock clicks] * (9.0 / 100.0))];
    $play_buffers_($cur_index_) set ts_ $ts;

    set cur_index_ [expr ($cur_index_ + 1) % $seg_length_];
    set interval [expr int(1000.0 / $playout_rate_) ];
    set next_play_after_handle_ [after $interval $self do_play];

    if {$encoder_ != ""} {
	$encoder_ recv $play_buffers_($cur_index_);
    }
}

FrameGen instproc load {} {
    $self instvar playing_flag_;

    if {$playing_flag_} {
	$self stop;
    }

    $self instvar loading_flag_;

    if {$loading_flag_} {
	return;
    }

    $self instvar in_agent_;

    set src_list [$in_agent_ set sources_];
    lappend src_list "";

    foreach s $src_list {
	if {$s != [$in_agent_ local] && [$s lastdata] != ""} {
	    if {[$self get_option in_ssrcid] == "*"} {
		break;
	    } elseif {[$self get_option in_ssrcid] == [$s srcid]} {
		break;
	    }
	}
    }
    
    if {$s == ""} {
	puts "No source on input";
	return;
    }
    $self instvar in_buffer_

    set in_buffer_ [new VidRep/JPEGCompressed];
    set decoder [new Module/VideoDecoder/JPEGToJPEGCompressed];

    $decoder set_frame_buffer $in_buffer_;
    $decoder set_callback "$self recv_frame";
    $decoder set agent_ $in_agent_;
    $decoder set src_ $s;

    $s handler $decoder;

    set loading_flag_ 1;

    $self instvar cur_index_ start_index_ load_src_;

    set load_src_ $s;

    set start_index_ $cur_index_;

    $self instvar seg_length_;

    .load configure -text "Loading...[expr ($cur_index_ - $start_index_ + $seg_length_) % $seg_length_]";
}

FrameGen instproc recv_frame {} {
    $self instvar in_buffer_ cur_index_ play_buffers_ seg_length_ start_index_;
    
    $play_buffers_($cur_index_) copy $in_buffer_;
    set cur_index_ [expr ($cur_index_ + 1) % $seg_length_];

    .load configure -text "Loading...[expr ($cur_index_ - $start_index_ + $seg_length_) % $seg_length_]";
    
    if {$cur_index_ == $start_index_} {
	$self stop_loading;
    }
}

FrameGen instproc stop_loading {} {
    $self instvar load_src_ in_buffer_ in_agent_
    $self instvar loading_flag_ start_index_;

    set d [new Module/VideoDecoder/Null];
    $d set agent_ $in_agent_;
    $d set src_ $load_src_;
    $load_src_ handler $d;
    unset load_src_;

    delete $in_buffer_;
    unset in_buffer_;

    unset start_index_;

    set loading_flag_ 0;

    $self instvar init_load_;
    
    if {!$init_load_} {
	set init_load_ 1;
    }

    .load configure -text "Load";
}
    
    

Class FrameGenVideoAgent -superclass VideoAgent

FrameGenVideoAgent instproc create_decoder {src} {
    set d [$src handler];
    if {$d != ""} {
	delete $d
    }

    set decoder [new Module/VideoDecoder/Null];

    $decoder set agent_ $self;
    $decoder set src_ $src;

    return $decoder;
}
    

Class AnnounceListenManager/FrameGenControl -superclass AnnounceListenManager

AnnounceListenManager/FrameGenControl instproc init {mtu spec fgen} {
    $self next $mtu $spec;
    $self instvar frame_gen_;

    set frame_gen_ $fgen;
}

AnnounceListenManager/FrameGenControl instproc recv_announcement {oaddr mesg length} {
    $self instvar frame_gen_;

    set cmd [lindex $mesg 0];

    switch -exact $cmd {
	set_rate {
	    set new_rate [lindex $mesg 1];
	    $frame_gen_ set_rate $new_rate;
	    .frame_rate set $new_rate;
	}
	incr_rate {
	    set old_rate [$frame_gen_ set playout_rate_];
	    set incr_factor [lindex $mesg 1];
	    set new_rate [expr $old_rate * $incr_factor];
	    $frame_gen_ set_rate $new_rate;
	    .frame_rate set $new_rate;
	}
	play {
	    $frame_gen_ play;
	}
	stop {
	    $frame_gen_ stop;
	}
	load {
	    $frame_gen_ load;
	}
	quit {
	    exit;
	}
    }
}


set app [new FrameGen $argv];
